@@ -1,14 +1,33 @@
 from enum import Enum
-from typing import Any, List, Optional
-
-import pandas as pd
+from typing import TYPE_CHECKING, Any, List, Optional
 
 from pydantic import BaseModel, Field
 
-from intugle.adapters.types.duckdb.models import DuckdbConfig
 
-# FIXME load dynamically
-DataSetData = pd.DataFrame | DuckdbConfig
+# Dynamic type registration - DataSetData is now built from registered adapters
+# This avoids hardcoding specific config types
+def get_dataset_data_type() -> type:
+    """
+    Returns the dynamically constructed DataSetData Union type.
+    This function retrieves the type from AdapterFactory after all adapters have been registered.
+    """
+    from intugle.adapters.factory import AdapterFactory
+    return AdapterFactory.get_dataset_data_type()
+
+
+# For type hints that need DataSetData before runtime, we use a forward reference
+if TYPE_CHECKING:
+    import pandas as pd
+
+    from intugle.adapters.types.databricks.models import DatabricksConfig
+    from intugle.adapters.types.duckdb.models import DuckdbConfig
+    from intugle.adapters.types.postgres.models import PostgresConfig
+    from intugle.adapters.types.snowflake.models import SnowflakeConfig
+
+    DataSetData = pd.DataFrame | DuckdbConfig | SnowflakeConfig | DatabricksConfig | PostgresConfig
+else:
+    # At runtime, this is dynamically determined
+    DataSetData = Any
 
 
 class ProfilingOutput(BaseModel):
@@ -49,7 +68,7 @@ class ColumnProfile(BaseModel):
     )
     datatype_l2: Optional[str] = Field(
         default=None,
-        description="The inferred data type category (dimension/measure) for the column, based on the datatype l1 results",  # noqa: E501
+        description="The inferred data type category (dimension/measure) for the column, based on the datatype l1 results",
     )
     business_glossary: Optional[str] = Field(
         default=None, description="The business glossary entry for the column, if available."
