@@ -1,6 +1,6 @@
 import time
 
-from typing import Any, Optional
+from typing import TYPE_CHECKING, Any, Optional
 
 import numpy as np
 import pandas as pd
@@ -10,13 +10,30 @@ from intugle.adapters.adapter import Adapter
 from intugle.adapters.factory import AdapterFactory
 from intugle.adapters.models import (
     ColumnProfile,
+    DataSetData,
     ProfilingOutput,
 )
 from intugle.adapters.utils import convert_to_native
+from intugle.core import settings
 from intugle.core.utilities.processing import string_standardization
 
+if TYPE_CHECKING:
+    from intugle.analysis.models import DataSet
+
 
 class PandasAdapter(Adapter):
+    @property
+    def database(self) -> Optional[str]:
+        return None
+
+    @property
+    def schema(self) -> Optional[str]:
+        return None
+    
+    @property
+    def source_name(self) -> str:
+        return settings.PROFILES.get("pandas", {}).get("name", "my_pandas_source")
+    
     def profile(self, data: pd.DataFrame, _: str) -> ProfilingOutput:
         """
         Generates a profile of a pandas DataFrame.
@@ -151,16 +168,48 @@ class PandasAdapter(Adapter):
         ...
         # duckdb.sql(f"CREATE TABLE {table_name} AS SELECT * FROM data")
     
-    def execute():
-        ...
+    def execute(self, query: str):
+        raise NotImplementedError("Execute is not supported for PandasAdapter yet.")
     
     def to_df(self, data):
         return data
 
+    def to_df_from_query(self, query: str) -> pd.DataFrame:
+        raise NotImplementedError("to_df_from_query is not supported for PandasAdapter yet.")
+
+    def create_table_from_query(
+        self, table_name: str, query: str, materialize: str = "view", **kwargs
+    ):
+        raise NotImplementedError(
+            "create_table_from_query is not supported for PandasAdapter yet."
+        )
+
+    def create_new_config_from_etl(self, etl_name: str) -> "DataSetData":
+        raise NotImplementedError("create_new_config_from_etl is not supported for PandasAdapter yet.")
+
+    def deploy_semantic_model(self, semantic_model_dict: dict, **kwargs):
+        """Deploys a semantic model to the target system."""
+        raise NotImplementedError("Deployment is not supported for the PandasAdapter.")
+
+    def intersect_count(self, table1: "DataSet", column1_name: str, table2: "DataSet", column2_name: str) -> int:
+        df1 = table1.data
+        df2 = table2.data
+        
+        if not isinstance(df1, pd.DataFrame) or not isinstance(df2, pd.DataFrame):
+            raise TypeError("Data for intersect_count must be pandas DataFrames for PandasAdapter.")
+
+        col1_unique = df1[column1_name].dropna().unique()
+        col2_unique = df2[column2_name].dropna().unique()
+
+        # Using numpy's intersect1d for performance with large arrays
+        intersection = np.intersect1d(col1_unique, col2_unique, assume_unique=True)
+        
+        return len(intersection)
+
 
 def can_handle_pandas(data: Any) -> bool:
     return isinstance(data, pd.DataFrame)
 
 
 def register(factory: AdapterFactory):
-    factory.register("pandas", can_handle_pandas, PandasAdapter)
+    factory.register("pandas", can_handle_pandas, PandasAdapter, pd.DataFrame)
