@@ -1,9 +1,13 @@
 import logging
 
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Optional
 
 from langchain.chat_models import init_chat_model
-from langchain.output_parsers import ResponseSchema, StructuredOutputParser
+from langchain.output_parsers import (
+    ResponseSchema,
+    RetryWithErrorOutputParser,
+    StructuredOutputParser,
+)
 from langchain.prompts import BaseChatPromptTemplate, ChatPromptTemplate
 from langchain_core.rate_limiters import InMemoryRateLimiter
 
@@ -26,7 +30,7 @@ class ChatModelLLM:
 
     def __init__(
         self,
-        model_name: str,
+        model_name: Optional[str] = None,
         response_schemas: list[ResponseSchema] = None,
         output_parser=StructuredOutputParser,
         prompt_template=ChatPromptTemplate,
@@ -35,16 +39,23 @@ class ChatModelLLM:
         *args,
         **kwargs,
     ):
-        self.model: BaseChatModel = init_chat_model(
-            model_name, max_retries=self.MAX_RETRIES, rate_limiter=self._get_rate_limiter(), **config
-        )  # llm model
+        if settings.CUSTOM_LLM_INSTANCE:
+            self.model: "BaseChatModel" = settings.CUSTOM_LLM_INSTANCE
+        elif model_name:
+            self.model: "BaseChatModel" = init_chat_model(
+                model_name, max_retries=self.MAX_RETRIES, rate_limiter=self._get_rate_limiter(), **config
+            )
+        else:
+            raise ValueError("Either 'settings.CUSTOM_LLM_INSTANCE' must be set or 'LLM_PROVIDER' must be provided.")
 
         self.parser: StructuredOutputParser = output_parser  # the output parser
 
         self.prompt_template: BaseChatPromptTemplate = prompt_template  # prompt template
 
         self.output_parser = (
-            self.__output_parser_builder__(response_schemas=response_schemas) if response_schemas is not None else None
+            self.__output_parser_builder__(response_schemas=response_schemas)
+            if response_schemas is not None
+            else None
         )  # the built output parser
 
         self.format_instructions = (
@@ -62,8 +73,11 @@ class ChatModelLLM:
         """
         for building the corresponding output paraser from the given ResponseSchema
         """
-        output_parser = self.parser.from_response_schemas(response_schemas=response_schemas)
-        return output_parser
+        parser = self.parser.from_response_schemas(response_schemas=response_schemas)
+        retry_parser = RetryWithErrorOutputParser.from_llm(
+            parser=parser, llm=self.model, max_retries=self.MAX_RETRIES
+        )
+        return retry_parser
 
     @classmethod
     def _get_rate_limiter(cls):
@@ -76,8 +90,6 @@ class ChatModelLLM:
             )
         return rate_limiter
 
-    def message_builder(): ...
-
     def invoke(self, *args, **kwargs):
         """
         The final invoke method that takes any arguments that is to be finally added in the prompt message and invokes the llm call.
@@ -85,7 +97,10 @@ class ChatModelLLM:
 
         sucessfull_parsing = False
 
-        messages = self.llm_prompt.format(format_instructions=self.format_instructions, **kwargs)
+        prompt_value = self.llm_prompt.format_prompt(
+            format_instructions=self.format_instructions, **kwargs
+        )
+        messages = prompt_value.to_messages()
         _message = messages
         response = ""
 
@@ -112,7 +127,7 @@ class ChatModelLLM:
             if self.output_parser is not None:
                 # try to parse the content as dict
                 raw_response = response
-                response = self.output_parser.parse(response)
+                response = self.output_parser.parse_with_prompt(response, prompt_value)
                 sucessfull_parsing = True
                 return response, sucessfull_parsing, raw_response
         except Exception as ex:
@@ -123,6 +138,8 @@ class ChatModelLLM:
 
     @classmethod
     def get_llm(cls, model_name: str, llm_config: dict = {}):
+        if settings.CUSTOM_LLM_INSTANCE:
+            return settings.CUSTOM_LLM_INSTANCE
         return init_chat_model(
             model_name, max_retries=cls.MAX_RETRIES, rate_limiter=cls._get_rate_limiter(), **llm_config
         )
