@@ -6,9 +6,11 @@ from typing import List, Optional, Tuple
 
 import pandas as pd
 
+from langchain.output_parsers import OutputFixingParser
 from langchain_core.messages import HumanMessage
-from langchain_core.output_parsers import JsonOutputParser
+from langchain_core.output_parsers import PydanticOutputParser
 from langchain_core.prompts import PromptTemplate
+from langgraph.errors import GraphRecursionError
 from tqdm import tqdm
 
 from intugle.analysis.models import DataSet
@@ -16,7 +18,6 @@ from intugle.core import settings
 from intugle.core.llms.chat import ChatModelLLM
 from intugle.core.pipeline.link_prediction.utils import (
     dtype_check,
-    extract_innermost_dict,
     linkage,
     preprocess_profiling_df,
 )
@@ -103,14 +104,15 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
             else llm
         )
 
-        self.parser_linkages = JsonOutputParser(pydantic_object=linkage)
+        pydantic_parser = PydanticOutputParser(pydantic_object=linkage)
+        self.parser_linkages = OutputFixingParser.from_llm(
+            llm=self.llm, parser=pydantic_parser
+        )
 
         self.prompt_link = PromptTemplate(
             template=self.PROMPT_TEMPLATE,
             input_variables=["current_attempt", "previous_attempts", "table_info"],
-            partial_variables={
-                "format_instructions": self.parser_linkages.get_format_instructions()
-            },
+            partial_variables={"format_instructions": self.parser_linkages.get_format_instructions()},
         )
 
         self.link_identifier_llm = self.prompt_link | self.llm | self.parser_linkages
@@ -132,9 +134,7 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
                 columns_required=self.PROFILING_COLUMNS_REQUIRED,
                 mapping_dtypes_to_sql=True,
             )
-            for (table_name,), gbydata in self.profiling_data.groupby(
-                ["table_name"]
-            )
+            for (table_name,), gbydata in self.profiling_data.groupby(["table_name"])
         }
 
         self.graph = self.__workflow_builder__()
@@ -174,9 +174,7 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
             error_msg = state["error_msg"]
 
         tab1, tab2 = input_text.split(" & ")
-        table_details = (
-            self.table_ddl_statements[tab1] + "\n\n" + self.table_ddl_statements[tab2]
-        )
+        table_details = self.table_ddl_statements[tab1] + "\n\n" + self.table_ddl_statements[tab2]
 
         if iteration == 0:
             current_attempt = 1
@@ -191,55 +189,50 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
         log.info(log_msg)
 
         try:
-            potential_link = self.link_identifier_llm.invoke(
-                {
-                    "current_attempt": "Attempt " + str(current_attempt),
-                    "table_info": table_details,
-                    "previous_attempts": previous_attempt_details,
-                }
-            )
+            potential_link = self.link_identifier_llm.invoke({
+                "current_attempt": "Attempt " + str(current_attempt),
+                "table_info": table_details,
+                "previous_attempts": previous_attempt_details,
+            })
         except Exception:
             log_msg = "[!] Killing LLM run due to halucinations in output"
             self.logs.append(log_msg)
             self.status = Status.HALLUCINATED_INVOKE
             log.info(log_msg)
 
-            # log.info('[!] Killing LLM run due to halucinations in output')
-            potential_link = {
-                "table1": "NA",
-                "column1": "NA",
-                "table2": "NA",
-                "column2": "NA",
-            }
+            potential_link = linkage(
+                table1="NA",
+                column1="NA",
+                table2="NA",
+                column2="NA",
+            )
         iteration = current_attempt
-
-        # Recursively search for the innermost dictionary
-        potential_link_innermost_dict = extract_innermost_dict(potential_link)
+        potential_link_dict = potential_link.model_dump()
 
         # checking for a possibility of recurcive loop in llm
         if iteration > 1:
             previous_potential_link = state["potential_link"]
-            if previous_potential_link == potential_link_innermost_dict:
-                log_msg = f"[!] Killing LLM run due to possibility of recurssion: {potential_link_innermost_dict}"
+            if previous_potential_link == potential_link_dict:
+                log_msg = f"[!] Killing LLM run due to possibility of recurssion: {potential_link_dict}"
                 self.status = Status.HALLUCINATED_RECURSION
                 log.info(log_msg)
                 self.logs.append(log_msg)
-                potential_link_innermost_dict = {
+                potential_link_dict = {
                     "table1": "NA",
                     "column1": "NA",
                     "table2": "NA",
                     "column2": "NA",
                 }
                 return {
-                    "potential_link": potential_link_innermost_dict,
+                    "potential_link": potential_link_dict,
                     "iteration": iteration,
                 }
 
-        log_msg = "[*] This is the link:" + str(potential_link_innermost_dict)
+        log_msg = "[*] This is the link:" + str(potential_link_dict)
         log.info(log_msg)
         self.logs.append(log_msg)
 
-        return {"potential_link": potential_link_innermost_dict, "iteration": iteration}
+        return {"potential_link": potential_link_dict, "iteration": iteration}
 
     def single_link_table_column_name_checker(self, state):
         """
@@ -260,9 +253,7 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
         final_error_msg_list = state["error_msg"]
         log.info("chk1")
         # check if table names present
-        stat_foreign_key = self.profiling_data.loc[
-            self.profiling_data["table_name"] == potential_link["table1"]
-        ]
+        stat_foreign_key = self.profiling_data.loc[self.profiling_data["table_name"] == potential_link["table1"]]
 
         if stat_foreign_key.shape[0] == 0:
             log.info("chk2")
@@ -282,15 +273,15 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
 
         log.info("chk3")
 
-        stat_foreign_key = stat_foreign_key.loc[
-            stat_foreign_key["column_name"] == potential_link["column1"],
-        ]
+        stat_foreign_key = stat_foreign_key.loc[stat_foreign_key["column_name"] == potential_link["column1"],]
 
         if stat_foreign_key.shape[0] == 0:
             log.info("chk4")
             table_name = potential_link["table1"]
             missing_column = potential_link["column1"]
-            error_msg = f"{missing_column} is not present in {table_name}. Check for any spelling errors in the column name."
+            error_msg = (
+                f"{missing_column} is not present in {table_name}. Check for any spelling errors in the column name."
+            )
             try:
                 final_error_msg_list.append(error_msg)
             except Exception:
@@ -306,9 +297,7 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
         # Referrenced table #########################3
         log.info("chk5")
         # check if table names present
-        stat_referrence_key = self.profiling_data.loc[
-            self.profiling_data["table_name"] == potential_link["table2"]
-        ]
+        stat_referrence_key = self.profiling_data.loc[self.profiling_data["table_name"] == potential_link["table2"]]
 
         if stat_referrence_key.shape[0] == 0:
             log.info("chk6")
@@ -327,14 +316,14 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
             return {"error_msg": final_error_msg_list, "if_error": True}
         log.info("chk7")
         # check if column names present
-        stat_referrence_key = stat_referrence_key.loc[
-            stat_referrence_key["column_name"] == potential_link["column2"]
-        ]
+        stat_referrence_key = stat_referrence_key.loc[stat_referrence_key["column_name"] == potential_link["column2"]]
         if stat_referrence_key.shape[0] == 0:
             log.info("chk8")
             table_name = potential_link["table2"]
             missing_column = potential_link["column2"]
-            error_msg = f"{missing_column} is not present in {table_name}. Check for any spelling errors in the column name."
+            error_msg = (
+                f"{missing_column} is not present in {table_name}. Check for any spelling errors in the column name."
+            )
             try:
                 final_error_msg_list.append(error_msg)
             except Exception:
@@ -460,9 +449,7 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
 
         # Calculate the max uniqueness for the referenced key table
 
-        referrenced_key_datatype = stat_referrence_key["datatype_l1"].values[
-            0
-        ]
+        referrenced_key_datatype = stat_referrence_key["datatype_l1"].values[0]
 
         if dtype_check(dtype1=foreign_key_datatype, dtype2=referrenced_key_datatype):
             # log.info('All Good No error found')
@@ -489,6 +476,92 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
 
             return {"error_msg": final_error_msg_list, "if_error": True}
 
+    def single_link_intersection_checker(self, state):
+        """
+        Checks intersection of values between columns
+
+        Args:
+            state (dict): The current graph state
+
+        Returns:
+            state (dict): New keys added to state, error_msg, error message if intersection between both the columns is low
+        """
+        log_msg = "---CHECKING FOR INTERSECTION BETWEEN COLUMNS---"
+        log.info(log_msg)
+        self.logs.append(log_msg)
+
+        potential_link = state["potential_link"]
+        final_error_msg_list = state["error_msg"]
+
+        table1_name = potential_link["table1"]
+        column1_name = potential_link["column1"]
+        table2_name = potential_link["table2"]
+        column2_name = potential_link["column2"]
+
+        count_distinct_col1 = self.profiling_data.loc[
+            (self.profiling_data.table_name == table1_name)
+            & (self.profiling_data.column_name == column1_name)
+        ]["distinct_count"].values[0]
+        count_distinct_col2 = self.profiling_data.loc[
+            (self.profiling_data.table_name == table2_name)
+            & (self.profiling_data.column_name == column2_name)
+        ]["distinct_count"].values[0]
+
+        # Use adapter from one of the datasets to execute query
+        adapter = self.table1.adapter
+
+        try:
+            intersect_count = adapter.intersect_count(
+                table1=self.table1,
+                column1_name=column1_name,
+                table2=self.table2,
+                column2_name=column2_name
+            )
+        except Exception as e:
+            log.error(f"Error executing intersection query: {e}")
+            error_msg = f"Could not calculate intersection between {table1_name}.{column1_name} and {table2_name}.{column2_name}."
+            try:
+                final_error_msg_list.append(error_msg)
+            except AttributeError:
+                final_error_msg_list = [error_msg]
+            return {"error_msg": final_error_msg_list, "if_error": True}
+
+        intersect_ratio_from_col = intersect_count / count_distinct_col1 if count_distinct_col1 > 0 else 0
+        intersect_ratio_to_col = intersect_count / count_distinct_col2 if count_distinct_col2 > 0 else 0
+
+        if intersect_count == 0:
+            error_msg = f"The intersection between {column1_name} column in table {table1_name} with {column2_name} in table {table2_name} resulted in zero rows"
+            try:
+                final_error_msg_list.append(error_msg)
+            except AttributeError:
+                final_error_msg_list = [error_msg]
+            log.info(error_msg)
+            self.logs.append(error_msg)
+            return {"error_msg": final_error_msg_list, "if_error": True}
+
+        elif max(intersect_ratio_from_col, intersect_ratio_to_col) < settings.INTERSECT_RATIO_THRESHOLD:
+            error_msg1 = f"Only {intersect_ratio_from_col * 100:.2f} percent of values in {column1_name} in table {table1_name} matched with {column2_name} in table {table2_name}. "
+            error_msg2 = f"Only {intersect_ratio_to_col * 100:.2f} percent of values in {column2_name} in table {table2_name} matched with {column1_name} in table {table1_name}."
+            error_msg = error_msg1 + error_msg2
+            try:
+                final_error_msg_list.append(error_msg)
+            except AttributeError:
+                final_error_msg_list = [error_msg]
+            log.info(error_msg)
+            self.logs.append(error_msg)
+            return {"error_msg": final_error_msg_list, "if_error": True}
+        else:
+            log_msg = "All Good No error found"
+            log.info(log_msg)
+            self.logs.append(log_msg)
+            return {
+                "if_error": False,
+                "intersect_count": intersect_count,
+                "intersect_ratio_from_col": round(intersect_ratio_from_col, 3),
+                "intersect_ratio_to_col": round(intersect_ratio_to_col, 3),
+                "accuracy": round(max(intersect_ratio_from_col, intersect_ratio_to_col), 3),
+            }
+
     def link_check_router(self, state):
         """
         Route the link checks
@@ -560,12 +633,9 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
             "single_link_table_column_name_checker",
             self.single_link_table_column_name_checker,
         )  # table column name checker
-        workflow.add_node(
-            "single_link_uniqueness_checker", self.single_link_uniqueness_checker
-        )  # uniqueness
-        workflow.add_node(
-            "single_link_datatype_checker", self.single_link_datatype_checker
-        )  # datatype
+        workflow.add_node("single_link_uniqueness_checker", self.single_link_uniqueness_checker)  # uniqueness
+        workflow.add_node("single_link_datatype_checker", self.single_link_datatype_checker)  # datatype
+        workflow.add_node("single_link_intersection_checker", self.single_link_intersection_checker)
 
         # Adding edges
         workflow.add_edge(START, "link_identifier")
@@ -601,6 +671,15 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
         workflow.add_conditional_edges(
             "single_link_datatype_checker",
             self.error_checker,
+            {
+                "Yes": "link_identifier",
+                "No": "single_link_intersection_checker",
+            },
+        )
+
+        workflow.add_conditional_edges(
+            "single_link_intersection_checker",
+            self.error_checker,
             {
                 "Yes": "link_identifier",
                 "No": END,
@@ -613,13 +692,10 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
         return graph
 
     def __post_processing__(self, data: pd.DataFrame) -> pd.DataFrame:
-        def process(result: str):
-            try:
-                result = eval(result) if isinstance(result, str) else result
-                return result if result["table1"] != "NA" else ""
-            except Exception as ex:
-                log.info(f"[!] Error while parsing: {ex}")
-                return ""
+        def process(result):
+            if isinstance(result, dict) and result.get("table1") != "NA":
+                return result
+            return ""
 
         data["links"] = data.links.apply(process)
         res = data.links.tolist()
@@ -638,14 +714,14 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
         return res
 
     def __graph_invoke__(self, table1: DataSet, table2: DataSet) -> dict:
+        self.table1 = table1
+        self.table2 = table2
         final_output = {}
         final_output["table1"] = table1.name
         final_output["table2"] = table2.name
 
         start_time = time.time()
-        input_message = HumanMessage(
-            content=f"{table1.name} & {table2.name}"
-        )
+        input_message = HumanMessage(content=f"{table1.name} & {table2.name}")
         init_data = {
             "input_text": input_message.content,
             "iteration": None,
@@ -659,22 +735,21 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
                 stream_mode="values",
                 config={
                     "recursion_limit": 20,
-                    "metadata": {
-                        "table_combo": tuple(
-                            sorted([table1.name, table2.name])
-                        )
-                    },
+                    "metadata": {"table_combo": tuple(sorted([table1.name, table2.name]))},
                 },
             ):
                 for key in [list(event.keys())[-1]]:
                     log.info(f"Finished running: {key}:")
 
+        except GraphRecursionError as ex:
+            log.warning(f"[!] Graph went into recursion loop when running for {table1.name} <=> {table2.name}")
+            event["status"] = Status.HALLUCINATED_RECURSION
+            event["potential_link"] = "NA"
+            event["error_msg"] = ex
+
         except Exception as ex:
             import traceback
-
-            log.error(
-                f"[!] Error while running for {table1.name} <=> {table2.name}: Reason {traceback.format_exc()}"
-            )
+            log.error(f"[!] Error while running for {table1.name} <=> {table2.name}: Reason {traceback.format_exc()}")
             event["status"] = Status.HALLUCINATED_RECURSION
             event["potential_link"] = "NA"
             event["error_msg"] = ex
@@ -687,6 +762,14 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
         log.info(f"[*] link between tables: {event['potential_link']}")
         potential_link = event["potential_link"]
 
+        if isinstance(potential_link, dict) and potential_link.get("table1") != "NA":
+            potential_link["intersect_count"] = event.get("intersect_count")
+            potential_link["intersect_ratio_from_col"] = event.get(
+                "intersect_ratio_from_col"
+            )
+            potential_link["intersect_ratio_to_col"] = event.get("intersect_ratio_to_col")
+            potential_link["accuracy"] = event.get("accuracy")
+
         final_output["links"] = potential_link
         final_output["Runtime_secs"] = runtime
         # Logs monitoring
@@ -701,22 +784,17 @@ Based on your evaluation of the data and metadata, please proceed to attempt ide
         pb = tqdm(assets)
         for _, (table1, table2) in enumerate(pb):
             runs = 1
-            pb.set_description(
-                desc=f"[*] {table1.name} <==> {table2.name} Runs:{runs}"
-            )
+            pb.set_description(desc=f"[*] {table1.name} <==> {table2.name} Runs:{runs}")
 
             final_output = self.__graph_invoke__(table1, table2)
 
             # If status came out as Hallucinated retry one more time
             while (
-                final_output["status"]
-                in (Status.HALLUCINATED_INVOKE, Status.HALLUCINATED_RECURSION)
+                final_output["status"] in (Status.HALLUCINATED_INVOKE, Status.HALLUCINATED_RECURSION)
                 and runs < self.HALLUCINATIONS_MAX_RETRY
             ):
                 runs += 1
-                log.info(
-                    f"[*] Hallucinated for {table1.name} <==> {table2.name} ... Retry no {runs} "
-                )
+                log.info(f"[*] Hallucinated for {table1.name} <==> {table2.name} ... Retry no {runs} ")
                 final_output = self.__graph_invoke__(table1=table1, table2=table2)
 
             results.append(final_output)
