@@ -4,12 +4,12 @@ import os
 
 from functools import lru_cache
 from pathlib import Path
-from typing import Optional
+from typing import Any, Optional
 
 from dotenv import load_dotenv
 from pydantic_settings import BaseSettings, SettingsConfigDict
 
-from intugle.core.utilities.configs import load_model_configuration
+from intugle.core.utilities.configs import load_model_configuration, load_profiles_configuration
 
 load_dotenv(dotenv_path=Path(__file__).resolve().parent.parent / ".env")
 
@@ -40,6 +40,9 @@ class Settings(BaseSettings):
     DI_MODEL_VERSION: str = "13052023"
 
     PROJECT_BASE: str = create_project_base_if_not_exists()
+    PROFILES_PATH: str = os.path.join(os.getcwd(), "profiles.yml")
+
+    PROFILES: dict = load_profiles_configuration(PROFILES_PATH)
 
     MCP_SERVER_NAME: str = "intugle"
     MCP_SERVER_DESCRIPTION: str = "Data Tools for MCP"
@@ -57,27 +60,25 @@ class Settings(BaseSettings):
     L2_SAMPLE_LIMIT: int = 10
 
     # LLM CONFIGS
-    LLM_PROVIDER: str
+    LLM_PROVIDER: Optional[str] = None
     LLM_SAMPLE_LIMIT: int = 15
     STRATA_SAMPLE_LIMIT: int = 4
     MAX_RETRIES: int = 5
     SLEEP_TIME: int = 25
     ENABLE_RATE_LIMITER: bool = False
+    CUSTOM_LLM_INSTANCE: Optional[Any] = None
+    CUSTOM_EMBEDDINGS_INSTANCE: Optional[Any] = None
 
     # LP
+    RELATIONSHIPS_FILE: str = "__relationships__.yml"
     HALLUCINATIONS_MAX_RETRY: int = 2
     UNIQUENESS_THRESHOLD: float = 0.9
+    INTERSECT_RATIO_THRESHOLD: float = 0.9
 
     # DATETIME
     DATE_TIME_FORMAT_LIMIT: int = 25
     REMOVE_DATETIME_LP: bool = True
 
-    model_config = SettingsConfigDict(
-        env_file=f"{BASE_PATH}/.env",
-        env_file_encoding="utf-8",
-        extra="allow",
-        case_sensitive=True,
-    )
     L2_SAMPLE_LIMIT: int = 10
 
     # LLM CONFIGS
@@ -85,14 +86,6 @@ class Settings(BaseSettings):
     LLM_SAMPLE_LIMIT: int = 15
     STRATA_SAMPLE_LIMIT: int = 4
 
-    # LP
-    HALLUCINATIONS_MAX_RETRY: int = 2
-    UNIQUENESS_THRESHOLD: float = 0.9
-
-    # DATETIME
-    DATE_TIME_FORMAT_LIMIT: int = 25
-    REMOVE_DATETIME_LP: bool = True
-
     # Adapter
     POSTGRES_USER: Optional[str] = None
     POSTGRES_PASSWORD: Optional[str] = None
@@ -101,6 +94,20 @@ class Settings(BaseSettings):
     POSTGRES_PORT: int = 5432
     POSTGRES_SCHEMA: Optional[str] = "public"
 
+    # Vector
+    VECTOR_COLLECTION_NAME: str = os.getcwd().split('/')[-1]
+    QDRANT_URL: str = "http://localhost:6333"
+    QDRANT_API_KEY: Optional[str] = None
+    EMBEDDING_MODEL_NAME: str = "openai:ada"
+    TOKENIZER_MODEL_NAME: str = "cl100k_base"
+
+    model_config = SettingsConfigDict(
+        env_file=f"{BASE_PATH}/.env",
+        env_file_encoding="utf-8",
+        extra="allow",
+        case_sensitive=True,
+    )
+
 
 @lru_cache
 def get_settings() -> Settings:
