@@ -4,7 +4,7 @@ from pydantic import BaseModel, Field
 
 from .operators import Operators
 from .transformation import Transformations
-from .utils import get_formatted_dataset_field
+from .utils import get_formatted_dataset_field, get_formatted_name
 
 
 class QueryGeneratorModel(BaseModel):
@@ -53,7 +53,7 @@ class QueryGenerator:
 
                 renameCol = field.get("renameCol", None)
                 if renameCol:
-                    expr = f"{expr} as `{renameCol}`"
+                    expr = f"{expr} as {get_formatted_name(renameCol)}"
 
                 selectedFields.append(expr)
             selectedFieldsExpr = ", ".join(selectedFields)
@@ -70,7 +70,7 @@ class QueryGenerator:
         for _i in range(tbl_cnt):
             i = str(_i)
             if _i == 0:
-                join_query += f"`{joinFields[i]['dataset']}`"
+                join_query += f"{get_formatted_name(joinFields[i]['dataset'])}"
             else:
                 joining_cndns = []
                 for join_col in joinFields[i]["fields"]:
@@ -92,33 +92,34 @@ class QueryGenerator:
                     " "
                     + joinFields[i].get("join_type", "").upper()
                     + " JOIN "
-                    + f"`{joinFields[i]['dataset']}`"
+                    + f"{get_formatted_name(joinFields[i]['dataset'])}"
                     + " ON "
                     + str(joining_cndn)
                 )
         return join_query
 
-    def _filter(self, filterFields):
-        fields = filterFields.get("fields", [])
-        filterCols = []
+    @classmethod
+    def _filter(cls, filter_fields):
+        fields = filter_fields.get("fields", [])
+        filter_cols = []
         for _field in fields:
             _fields = _field.get("fields", None)
             if _fields:
-                expr = self._filter(self, _field)
+                expr = cls._filter(_field)
                 expr = f"({expr})"
             else:
                 expr = Operators.operator_factory(_field)
-            filterCols.append(expr)
-        return " {} ".format(filterFields["condition"].upper()).join(filterCols)
+            filter_cols.append(expr)
+        return " {} ".format(filter_fields["condition"].upper()).join(filter_cols)
 
     @classmethod
     def get_filter_expr(cls, filterFields):
         # Filter Expression
         filterExpr = ""
         if filterFields != {}:
-            filterCols = cls._filter(cls, filterFields)
-            if filterCols:
-                filterExpr = filterExpr + " WHERE " + filterCols
+            filter_cols = cls._filter(filterFields)
+            if filter_cols:
+                filterExpr = filterExpr + " WHERE " + filter_cols
         return filterExpr
 
     @classmethod
@@ -132,7 +133,7 @@ class QueryGenerator:
                 # if _dataset is not None:
                 #     _dataset_field = get_formatted_dataset_field(_dataset, _field)
                 # else:
-                #     _dataset_field = f"`{_field}`"
+                #     _dataset_field = f"{_field}"
 
                 # _func = field.get("function", None)
                 # if _func is not None:
@@ -140,7 +141,7 @@ class QueryGenerator:
                 #     _dataset_field = f"{_func}({_dataset_field})"
                 renameCol = field.get("renameCol", None)
                 if renameCol is not None:
-                    _dataset_field = f"`{renameCol}`"
+                    _dataset_field = get_formatted_name(renameCol)
                 else:
                     _dataset_field = field["sql_code"]
                 groupbyCols.append(_dataset_field)
@@ -163,7 +164,7 @@ class QueryGenerator:
             for sortField in sortFields:
                 renameCol = sortField.get("renameCol", None)
                 if renameCol is not None:
-                    sortCol = f"`{renameCol}`"
+                    sortCol = get_formatted_name(renameCol)
                 else:
                     sortCol = get_formatted_dataset_field(
                         sortField["dataset"], sortField["field"]
@@ -200,7 +201,7 @@ class QueryGenerator:
 
         # From dataset
         # if "from" in config:
-        #     sqlQuery = sqlQuery + " FROM " + f"`{config['from']}`"
+        #     sqlQuery = sqlQuery + " FROM " + f"{config['from']}"
 
         if config.join != {}:
             sqlQuery += cls.get_join_expr(config.join)
