@@ -27,8 +27,9 @@ from ..models.models import (
 )
 
 SEPERATOR = "____"
-SQL_CODE_REGEX_PATTERN = r"@\{([\d\-]+)\}\[([\w\-\.\`]+)\]"
-CF_REGEX_PATTERN = r"([A-Za-z_\s]+)\(([^()]+)\)"
+SQL_CODE_REGEX_PATTERN = re.compile(r"@\{([\d\-]+)\}\[([\w\-\.\`]+)\]")
+CF_REGEX_PATTERN = re.compile(r"([A-Za-z_]+(?:\s+[A-Za-z_]+)*)(\s*)\(([^()]*?)\)")
+MAX_SQL_CODE_LENGTH = 10000
 
 
 class CTE:
@@ -399,13 +400,16 @@ class CTE:
         return sql_code
 
     def parse_function_fields(self, field_id: int, sql_code: str, fields={}):
+        if len(sql_code) > MAX_SQL_CODE_LENGTH:
+            return sql_code, {}
+
         multiple_asset_field = False
 
         matches = re.findall(CF_REGEX_PATTERN, sql_code)
 
         for match in matches:
-            func = match[0]
-            args = match[1]
+            func = match[0] + match[1]
+            args = match[2]
             _fields, _assets = self.__get_field_ids_asset_ids(args, fields)
             if len(_assets) == 1:
                 _id = [field_id, *_fields, *_assets]
