@@ -3,21 +3,14 @@ import json
 import logging
 import os
 
-from typing import Any, Dict, List, Self
+from typing import Any, Dict, List
 
 import pandas as pd
 import yaml
 
 from intugle.analysis.models import DataSet
-from intugle.analysis.pipeline import Pipeline
-from intugle.analysis.steps import (
-    ColumnProfiler,
-    DataTypeIdentifierL1,
-    DataTypeIdentifierL2,
-    KeyIdentifier,
-    TableProfiler,
-)
 from intugle.core import settings
+from intugle.core.console import console, warning_style
 from intugle.core.pipeline.link_prediction.lp import LinkPredictionAgentic
 from intugle.libs.smart_query_generator.utils.join import Join
 from intugle.models.resources.relationship import (
@@ -31,6 +24,11 @@ from .models import LinkPredictionResult, PredictedLink
 log = logging.getLogger(__name__)
 
 
+class NoLinksFoundError(Exception):
+    """Custom exception raised when no links are found to save."""
+    pass
+
+
 class LinkPredictor:
     """
     Analyzes a collection of datasets to predict column links between all
@@ -48,13 +46,6 @@ class LinkPredictor:
                         of DataSet objects.
         """
         self.datasets: Dict[str, DataSet] = {}
-        self._prerequisite_pipeline = Pipeline([
-            TableProfiler(),
-            ColumnProfiler(),
-            DataTypeIdentifierL1(),
-            DataTypeIdentifierL2(),
-            KeyIdentifier(),
-        ])
         self.links: list[PredictedLink] = []
 
         if isinstance(data_input, dict):
@@ -73,8 +64,7 @@ class LinkPredictor:
 
     def _run_prerequisites(self, dataset: DataSet):
         """Runs the prerequisite analysis steps on a given DataSet."""
-        for step in self._prerequisite_pipeline.steps:
-            step.analyze(dataset)
+        dataset.profile().identify_datatypes().identify_keys()
 
     def _initialize_from_dict(self, data_dict: Dict[str, Any]):
         """Creates and processes DataSet objects from a dictionary of raw dataframes."""
@@ -89,7 +79,7 @@ class LinkPredictor:
         for dataset in data_list:
             if not dataset.name:
                 raise ValueError("DataSet objects provided in a list must have a 'name' attribute.")
-            if "key" not in dataset.results:
+            if dataset.source.table.key is None:
                 print(f"Dataset '{dataset.name}' is missing key identification. Running prerequisite analysis...")
                 self._run_prerequisites(dataset)
             else:
@@ -102,7 +92,11 @@ class LinkPredictor:
         return f"{assets[0]}--{assets[1]}"
 
     def _predict_for_pair(
-        self, name_a: str, dataset_a: DataSet, name_b: str, dataset_b: DataSet
+        self,
+        name_a: str,
+        dataset_a: DataSet,
+        name_b: str,
+        dataset_b: DataSet,
     ) -> List[PredictedLink]:
         """
         Contains the core logic for finding links between TWO dataframes.
@@ -111,19 +105,19 @@ class LinkPredictor:
         table_combination = self._create_table_combination_id(name_a, name_b)
         if table_combination in self.already_executed_combo:
             log.warning(f"[!] Skipping already executed combination: {table_combination}")
-            return
+            return []
 
-        dataset_a_column_profiles = [col.model_dump() for col in dataset_a.results["column_profiles"].values()]
-        dataset_b_column_profiles = [col.model_dump() for col in dataset_b.results["column_profiles"].values()]
+        dataset_a_column_profiles = dataset_a.profiling_df
+        dataset_b_column_profiles = dataset_b.profiling_df
         profiling_data = pd.concat(
-            [pd.DataFrame(dataset_a_column_profiles), pd.DataFrame(dataset_b_column_profiles)], ignore_index=True
+            [dataset_a_column_profiles, dataset_b_column_profiles], ignore_index=True
         )
 
         primary_keys = []
-        if dataset_a.results.get("key"):
-            primary_keys.append((name_a, dataset_a.results["key"]))
-        if dataset_b.results.get("key"):
-            primary_keys.append((name_b, dataset_b.results["key"]))
+        if dataset_a.source.table.key:
+            primary_keys.append((name_a, dataset_a.source.table.key))
+        if dataset_b.source.table.key:
+            primary_keys.append((name_b, dataset_b.source.table.key))
 
         pipeline = LinkPredictionAgentic(
             profiling_data=profiling_data,
@@ -138,16 +132,39 @@ class LinkPredictor:
                 from_column=row["column1_name"],
                 to_dataset=row["table2_name"],
                 to_column=row["column2_name"],
+                intersect_count=row.get("intersect_count"),
+                intersect_ratio_from_col=row.get("intersect_ratio_from_col"),
+                intersect_ratio_to_col=row.get("intersect_ratio_to_col"),
+                accuracy=row.get("accuracy"),
             )
             for _, row in llm_result.iterrows()
         ]
         return pair_links
 
-    def predict(self, filename='relationships.yml', save: bool = False) -> Self:
+    def predict(self, filename: str = None, save: bool = False, force_recreate: bool = False) -> 'LinkPredictor':
         """
         Iterates through all unique pairs of datasets, predicts the links for
         each pair, and returns the aggregated results.
         """
+        if filename is None:
+            filename = settings.RELATIONSHIPS_FILE
+
+        relationships_file = os.path.join(settings.PROJECT_BASE, filename)
+
+        if not force_recreate and os.path.exists(relationships_file):
+            is_stale = False
+            relationships_mtime = os.path.getmtime(relationships_file)
+            for dataset in self.datasets.values():
+                dataset_yml = os.path.join(settings.PROJECT_BASE, f"{dataset.name}.yml")
+                if os.path.exists(dataset_yml) and os.path.getmtime(dataset_yml) > relationships_mtime:
+                    is_stale = True
+                    break
+            
+            if not is_stale:
+                console.print("Link predictions are up-to-date. Loading from cache.", style="green")
+                self.load_from_yaml(relationships_file)
+                return self
+
         all_links: List[PredictedLink] = []
         dataset_names = list(self.datasets.keys())
 
@@ -165,13 +182,21 @@ class LinkPredictor:
                 print("No links found for this pair.")
 
         self.links = all_links
+
+        if len(self.links) == 0:
+            console.print("No links found between any datasets.", style=warning_style)
+            return self
+
         if save:
             self.save_yaml(file_path=filename)
 
         return self
     
-    def get_links_df(self):
-        ...
+    def get_links_df(self) -> pd.DataFrame:
+        """Returns the predicted links as a pandas DataFrame."""
+        if not self.links:
+            return pd.DataFrame()
+        return pd.DataFrame([link.model_dump() for link in self.links])
     
     def show_graph(self):
         links = [link.relationship.link for link in self.links]
@@ -186,7 +211,7 @@ class LinkPredictor:
         file_path = os.path.join(settings.PROJECT_BASE, file_path)
 
         if len(self.links) == 0:
-            raise ValueError("No links found to save.")
+            raise NoLinksFoundError("No links found to save.")
 
         relationships = {"relationships": [json.loads(link.relationship.model_dump_json()) for link in self.links]}
 
@@ -194,6 +219,28 @@ class LinkPredictor:
         with open(file_path, "w") as file:
             yaml.dump(relationships, file, sort_keys=False, default_flow_style=False)
 
+    def load_from_yaml(self, file_path: str) -> None:
+        """Loads link predictions from a YAML file."""
+        with open(file_path, "r") as f:
+            data = yaml.safe_load(f)
+        
+        relationships = data.get("relationships", [])
+        loaded_links = []
+        for rel in relationships:
+            metrics = rel.get("profiling_metrics", {}) or {}
+            link = PredictedLink(
+                from_dataset=rel["source"]["table"],
+                from_column=rel["source"]["column"],
+                to_dataset=rel["target"]["table"],
+                to_column=rel["target"]["column"],
+                intersect_count=metrics.get("intersect_count"),
+                intersect_ratio_from_col=metrics.get("intersect_ratio_from_col"),
+                intersect_ratio_to_col=metrics.get("intersect_ratio_to_col"),
+                accuracy=metrics.get("accuracy"),
+            )
+            loaded_links.append(link)
+        self.links = loaded_links
+
 
 class LinkPredictionSaver:
     @classmethod
